@@ -86,6 +86,17 @@ bool cN2Emu::Init(int id, int romv)
 
 // -- cMapCore -----------------------------------------------------------------
 
+#define SETSIZE  0x02
+#define IMPORT_J 0x03
+#define IMPORT_A 0x04
+#define IMPORT_B 0x05
+#define IMPORT_C 0x06
+#define IMPORT_D 0x07
+#define EXPORT_A 0x0A
+#define EXPORT_B 0x0B
+#define EXPORT_C 0x0C
+#define EXPORT_D 0x0D
+
 class cMapCore {
 private:
   cBN x, y, s, j;
@@ -101,7 +112,7 @@ protected:
   void SetWordSize(int l) { wordsize=l; }
   void MakeJ(void);
   void MonMul(BIGNUM *o, BIGNUM *i1, BIGNUM *i2);
-  bool DoMap(int f, unsigned char *data, int l);
+  bool DoMap(int f, unsigned char *data=0, int l=0);
 public:
   cMapCore(void);
   };
@@ -115,11 +126,11 @@ void cMapCore::ImportReg(unsigned char reg, const unsigned char *in, int l)
 {
   l=(l?l:wordsize)<<3;
   switch(reg) {
-    case 0xF: J.GetLE(in,8); break;
-    case 0xA: A.GetLE(in,l); break;
-    case 0xB: B.GetLE(in,l); break;
-    case 0xC: C.GetLE(in,l); break;
-    case 0xD: D.GetLE(in,l); break;
+    case IMPORT_J: J.GetLE(in,8); break;
+    case IMPORT_A: A.GetLE(in,l); break;
+    case IMPORT_B: B.GetLE(in,l); break;
+    case IMPORT_C: C.GetLE(in,l); break;
+    case IMPORT_D: D.GetLE(in,l); break;
     default: PRINTF(L_GEN_DEBUG,"internal: nagramap import register not supported"); return;
     }
 }
@@ -129,10 +140,10 @@ void cMapCore::ExportReg(unsigned char reg, unsigned char *out, int l, bool BE)
   l=(l?l:wordsize)<<3;
   cBN *ptr;
   switch(reg) {
-    case 0xA: ptr=&A; break;
-    case 0xB: ptr=&B; break;
-    case 0xC: ptr=&C; break;
-    case 0xD: ptr=&D; break;
+    case EXPORT_A: ptr=&A; break;
+    case EXPORT_B: ptr=&B; break;
+    case EXPORT_C: ptr=&C; break;
+    case EXPORT_D: ptr=&D; break;
     default: PRINTF(L_GEN_DEBUG,"internal: nagramap export register not supported"); return;
     }
   if(!BE) ptr->PutLE(out,l);
@@ -202,11 +213,11 @@ bool cMapCore::DoMap(int f, unsigned char *data, int l)
     case 0x44: // add 64 bytes to SHA1 buffer
       RotateBytes(data,64);
       SHA1_Update(&sctx,data,64);
-      put_unaligned(sctx.h4,(unsigned int *)(data   ));
-      put_unaligned(sctx.h3,(unsigned int *)(data+4 ));
-      put_unaligned(sctx.h2,(unsigned int *)(data+8 ));
-      put_unaligned(sctx.h1,(unsigned int *)(data+12));
-      put_unaligned(sctx.h0,(unsigned int *)(data+16));
+      BYTE4_LE(data   ,sctx.h4);
+      BYTE4_LE(data+4 ,sctx.h3);
+      BYTE4_LE(data+8 ,sctx.h2);
+      BYTE4_LE(data+12,sctx.h1);
+      BYTE4_LE(data+16,sctx.h0);
       break;
     case 0x45: // add wordsize bytes to SHA1 buffer and finalize SHA result
       if(wordsize) {
@@ -410,12 +421,12 @@ void cNagra2::CreatePQ(const unsigned char *key, BIGNUM *p, BIGNUM *q)
   // expand IDEA-G key
   unsigned char idata[96];
   for(int i=11; i>=0; i--) {
-    const int off=i*8;
-    memcpy(&idata[off],&key[13],8);
-    idata[off]^=i;
-    idea.Decrypt(&idata[off],8,&ks,0);
-    for(int j=7; j>=0; j--) idata[off+j] ^= key[13+j];
-    idata[off]^=i;
+    unsigned char *d=&idata[i*8];
+    memcpy(d,&key[13],8);
+    *d^=i;
+    idea.Decrypt(d,8,&ks,0);
+    xxor(d,8,d,&key[13]);
+    *d^=i;
     }
   // Calculate P
   idata[0] |= 0x80;
@@ -438,7 +449,7 @@ bool cNagra2::Signature(const unsigned char *vkey, const unsigned char *sig, con
     idea.SetEncKey(buff,&ks);
     memcpy(buff,buff+8,8);
     idea.Encrypt(msg+i,8,buff+8,&ks,0);
-    for(int j=7; j>=0; j--) buff[j+8]^=msg[i+j];
+    xxor(&buff[8],8,&buff[8],msg+i);
     }
   buff[8]&=0x7F;
   return (memcmp(sig,buff+8,8)==0);
@@ -606,7 +617,7 @@ bool cSystemNagra2::ProcessECM(const cEcmInfo *ecm, unsigned char *data)
         break;
       case 0x00:
         i+=2; break;
-      case 0x13 ... 0x15:
+      case 0x13 ... 0x17:
         i+=4; break;
       case 0x30 ... 0x36:
       case 0xB0:
@@ -649,7 +660,7 @@ void cSystemNagra2::ProcessEMM(int pid, int caid, unsigned char *buffer)
 
   int keyset=(buffer[12]&0x03);
   int sel=(buffer[12]&0x10)<<2;
-  int rsasel=(id==0x4101 || id==0x4001) ? 0:sel; // D+ hack
+  int rsasel=(id==0x4101 || id==0x4001 || id==0x4102) ? 0:sel; // D+ hack
   int sigsel=(buffer[13]&0x80)>>1;
   cPlainKey *pk;
   cBN n;
@@ -723,11 +734,12 @@ void cSystemNagra2::ProcessEMM(int pid, int caid, unsigned char *buffer)
         i+=emmdata[i+1]+2+4;
         break;
       case 0xA6:
-        i+=emmdata[i+1]+1;
+        i+=15;
         break;
-      case 0x13: // Date
-      case 0x14:
-      case 0x15:
+      case 0xAE:
+        i+=11;
+        break;
+      case 0x13 ... 0x17: // Date
         i+=4;
         break;
       case 0xB0 ... 0xBF: // Update with ROM CPU code
