@@ -40,7 +40,7 @@
 #error "This version of ttxtsubs only works with vdr version >= 1.4.0!"
 #endif
 
-static const char *VERSION        = "0.0.5-KNE";
+static const char *VERSION        = "0.0.5-RRE";
 static const char *DESCRIPTION    = "Teletext subtitles";
 
 cTtxtsubsConf globals;
@@ -625,6 +625,7 @@ cMenuSetupTtxtsubs::cMenuSetupTtxtsubs(cPluginTtxtsubs *ttxtsubs, int doStore)
   mDoStore(doStore),
   mConf(globals)
 {
+  int tmp;
   char *buf = NULL;
   //static char *mainMenuAlts[] = {"off", "Display on/off", "4:3/Letterbox", "This menu"};
   // can't get it to store changes in file
@@ -722,7 +723,7 @@ cMenuSetupTtxtsubs::cMenuSetupTtxtsubs(cPluginTtxtsubs *ttxtsubs, int doStore)
   if (mRed < 0 || mRed >= numColorValues) mRed = 0;
   mTransparency = (mConf.mCustomColor & 0x0F000000) >> 24;
   if (mTransparency < 0 || mTransparency >= numColorValues) mTransparency = 0;
-  asprintf(&buf, "%s: --------------------------------------------", tr("Custom Color"));
+  tmp = asprintf(&buf, "%s: --------------------------------------------", tr("Custom Color"));
   cOsdItem *item = new cOsdItem(buf);
   item->SetSelectable(false);
   free(buf);
